# In this file we look at finite sequences s_i where every s_i is a sequence
# of integers of length k such that these integers strictly increase.


#############################################################################
##
#F IsValidNextStep( <start>, <step>, <last>, <depth>, <max> )
##
# Given a sequence <alpha> and step-sequnce <step> (which contains of zeroes
# and ones only), it will be tested if...
#  - <alpha>+<beta> (elementwise addition) is strictly increasing
#  - <alpha>+<step> can be a successor of <alpha> 
#  - <alpha>+<step> is still smaller than the aim sequence <beta>
#  - <alpha>+<beta> is (depending on the length of the sequence so far, given
#      by <depth>) not too small that there can be enough successors filled in.
# 
IsValidNextStep := function( alpha, add, beta, depth, max )
    local l, gamma;
    l := Length(alpha);
    gamma := alpha + add;
    return ( ForAll( [1..l-1], x -> gamma[x] < gamma[x+1] ) and
             ForAll( [1..l], x -> alpha[x] <= gamma[x] and gamma[x] <= alpha[x]+1 ) and
             ForAll( [1..l], x -> gamma[x] <= beta[x] ) and
             ForAll( [1..l], x -> beta[x] - gamma[x] <= max - depth +1)
             );
end;

IsValidLastStep := function( alpha, add, max )
    local l, beta;
    l := Length(alpha);
    beta := alpha + add;
    return ( ForAll( [1..l-1], x -> beta[x] < beta[x+1] ) and
             ForAll( [1..l], x -> alpha[x] <= beta[x] and beta[x] <= alpha[x]+1 ) and
             ForAll( [1..l], x ->  beta[x] <=  max[x] and  max[x] <=  beta[x]+1 ));
end;

FindSequenceNumberByRecursion := function( alpha, beta, count, depth, max)
    local iter, l, add, erg, i;
    l    := Length( alpha );
    iter := IteratorOfTuples( [0,1], l );
    if depth < max then
        while not IsDoneIterator( iter ) do
            add := NextIterator( iter );
            if IsValidNextStep( alpha, add, beta, depth, max ) then
                count := FindSequenceNumberByRecursion( alpha+add, beta, count, depth+1, max );
            else
            fi;
        od;
    else
        while not IsDoneIterator( iter ) do
            add := NextIterator( iter );
            if IsValidLastStep( alpha, add, beta ) then
                count := count + 1;
            fi;
        od;
    fi;
    return count;
end;

FindSequenceNumber := function( alpha, beta, n )
    return FindSequenceNumberByRecursion( alpha, beta, 0, 1, n-1);
end;

ExceptionalPrimesAssociative := function( n )
    local val;
    val := List( [1..n], x -> FindSequenceNumber( [1..x], [n-x+1..n], n ));
Display(val);
    val := Maximum( Flat( List( val, Factors) ) );
    val := Position( Primes, val );
    return Concatenation( [0], Primes{[1..val]} );
end;
