#  l: Number of elements of which the combination is taken
# av: Combinations that are known to yield no solution and can be avoided
# kn: combinations that are worked through and hence are known
#  j: Length of combinations that have to be built

CombinationsAvoid := function( l, av, kn, j )
    local av1, av2, tr, nw, i, k, t;

    # the trivial case
    if j = 1 then return List([1..l], x -> [x]); fi;
    
    # otherwise split avoid
    av1 := Filtered(av, x -> Length(x) = 1);
    av2 := Difference(av, av1);

    # get all relevant elements
    tr := Difference([1..l], Flat(av1));

    # build up new sets of length j
    nw := [];
    for i in [1..Length(kn)] do
        k := kn[i][j-1];
        t := List(Intersection(tr,[k+1..l]),x->Concatenation(kn[i],[x]));
        t := Filtered(t, x -> not ForAny(av2, y -> IsSubset(x,y)));
        Append(nw, t);
    od;

    return nw;
end;

OnComb := function( eqn, mat )
    return Set(List(eqn, x -> x*mat));
end;

OnCombSet := function( set, mat )
    return Set(List(set, x -> OnComb(x,mat)));
end;

OnNVecs := function( vec, mat )
    local new, j;
    new := vec*mat;
    j := First(new, x -> x <> 0*x);
    j := LeadingCoefficient(j);
    if j > 0 then return new; else return -new; fi;
end;

MaxDivisorsInt := function(n)
    if n = 1 then return []; fi;
    return List(Set(Factors(n)), x -> n/x);
end;

