##
## JordanBlockLengths for arbitrary finite fields F
##
JordanBlockLengths := function( g, F )
    local chr, min, fc, fm, ns, V, W, U, i, f, e, h, k, v, t, R;

    R := PolynomialRing(F, ["x_1"]);

    # char poly 
    chr := CharacteristicPolynomial(F, F, g);
    fc := Collected(Factors(R, chr));

    # min poly 
    min := MinimalPolynomial(F, g);
    fm := Factors(R, min);

    # set up
    ns := List( fc, x -> List( [1..x[2]], y -> 0 ) );
    V := g^0;

    # loop 
    for i in [1..Length(fc)] do

        # determine V_i
        f := fc[i][1];
        e := Length(Filtered(fm, x -> x = f)); 
        h := Value( f^e, g );
        W := TriangulizedNullspaceMat(h);
        k := InducedActionFactor( [g], W, [] )[1];

        # split cyclic
        while not IsBool(k) do 
            ns[i][e] := ns[i][e] + 1;
            if Length(k) = e * Degree(f) then 
                k := false;
            else
                h := Value( f^(e-1), k );
                v := First( k^0, x -> x*h <> 0*x );
                W := SpinUpCyclic( v, k, e*Degree(f) );
                U := BaseSteinitzVectors(k^0, W).factorspace;
                k := InducedActionFactor( [k], U, W )[1];
                t := Collected(Factors(R, MinimalPolynomial( F, k )));
                e := t[1][2];
            fi;
        od;
    od;
    return rec( blks := ns, degs := List(fc, x -> Degree(x[1])));
end;

NumberOfClass2AssocAlgebras := function ( n, q )
    local k, c, m;
    m := n^2;
    c := [  ];
    c[m] := 1;
    for k  in [ 1 .. m - 1 ]  do
        if not IsBound( c[k] )  then
            c[k] := CountOrbitsGL( n, q, k, TensorAction );
            c[m - k] := c[k];
        fi;
    od;
    return c;
end;

