
#############################################################################
##
#F IsJNFCombi( a, dg )
##
IsJNFCombi := function( a, dg )
    local i;
    for i in [2..Length(a)] do
        if dg[i-1] = dg[i] then 
            if a[i-1] > a[i] then return false; fi;
        fi;
    od;
    return true;
end; 

#############################################################################
##
#F LinearCombinations( dg, n )
##
LinearCombinations := function( dg, n )
    local l, a, d, b, i, m, j, c;
    l := Length(dg);
    a := [List(dg, x -> 1)];
    d := [];
    while Length(a) > 0 do 
        b := [];
        for i in [1..Length(a)] do
            m := Sum( List([1..l], x -> dg[x]*a[i][x]));
            if m = n then 
                Add(d, a[i]);
            elif m < n then 
                for j in [1..l] do
                    c := StructuralCopy(a[i]);
                    c[j] := c[j]+1;
                    Add(b, c);
                od;
            fi;
        od;
        a := ShallowCopy(b);
    od;
    return Filtered(d, x -> IsJNFCombi(x, dg));
end;

#############################################################################
##
#F IsJNFType(t)
##
IsJNFType := function( dg, s, t )
    local i;
    for i in [2..Length(t)] do
        if dg[i-1] = dg[i] and s[i-1] = s[i] then 
            if t[i-1] > t[i] then return false; fi;
        fi;
    od;
    return true;
end;

#############################################################################
##
#F JNFTypes( <n> ) 
##
JNFTypes := function( n )
    local dgs, tps, cps, dg, sg, c, s, r, t, i;

    # possible degrees 
    dgs := Concatenation( List([1..n], Partitions) );
    tps := [];
    cps := [];

    # add blocks to degrees
    for dg in dgs do
        sg := LinearCombinations(dg, n);
        for s in sg do
            r := List(s, Partitions);
            r := Cartesian(r);
            c := Length(tps)+1;
            for i in [1..Length(r)] do
                if IsJNFType( dg, s, r[i] ) then 
                    t := List([1..Length(dg)], x -> [r[i][x], dg[x]]);
                    if not t in tps then Add(tps, t); Add(cps, c); fi;
                fi;
            od;
        od;
    od;

    # turn into records
    for i in [1..Length(tps)] do
        if cps[i] = i then 
            tps[i] := rec( type := tps[i], ss := true );
        else
            tps[i] := rec( type := tps[i], ss := cps[i] );
        fi;
    od;

    # that's it
    return tps;
end;

#############################################################################
##
#F NumberOfIrreds( <n> ) 
##
NumberOfIrreds := function( n )
    local q;
    q := NameToIndet("q");
    if n = 1 then return q-1; fi;
    return Sum(List(DivisorsInt(n), d -> MoebiusMu(n/d)*q^d))/n;
end;

#############################################################################
##
#F MyBino( <a>, <b> ) 
##
MyBino := function( a, b )
    return Product(List([0..b-1], x -> a-x))/Factorial(b);
end;

#############################################################################
##
#F SymmCount( <type> )
##
SymmCount := function( type )
    local b, d, k, e, i, j, c;

    # set up
    b := List(type.type, x -> x[1]);
    d := List(type.type, x -> x[2]);
    k := Length(b);
    e := 1;

    # loop over degrees
    for i in Set(d) do
        j := Filtered([1..k], x -> d[x]=i);
        c := List( Collected(b{j}), x -> Factorial(x[2]) );
        e := e * Product(c);
    od;

    # store
    return e;
end;

#############################################################################
##
#F NrElementsOfType( <type> ) 
##
NrElementsOfType := function( type )
    local b, d, k, e, i, l, m, j, c, h;

    # set up
    b := List(type.type, x -> x[1]);
    d := List(type.type, x -> x[2]);
    k := Length(b);
    e := 1;

    # loop over degrees
    for i in Set(d) do

        # choose without duplication
        l := Length(Filtered(d, x -> x = i));
        m := NumberOfIrreds( i );
        e := e * Product([0..l-1], x -> (m-x));

        # within blocks avoid repetitions
        j := Filtered([1..k], x -> d[x]=i);
        c := List( Collected(b{j}), x -> Factorial(x[2]) );
        e := e / Product(c);
    od;

    # store
    return e;
end;

#############################################################################
##
#F HallSumByPartition( <l>, <q> ) 
##
HallSumByPartition := function( l, q )
    local s, i, n;
    s := 1;
    for i in [1..Length(l)] do
        if i < Length(l) then 
            n := l[i]-l[i+1];
        else
            n := l[i];
        fi;
        if not n = 0 then 
            s := s * Product(List([1..n], x -> (1-(1/q)^x)));
        fi;
    od;
    return s;
end; 

#############################################################################
##
#F CentralizerOfBlock( <bl>, <dg> ) 
##
#
CentralizerOfBlock := function( bl, dg )
    local qq, l, i, k, a, b, s;

    # set up 
    qq := NameToIndet("q")^dg;
    
    # the irreducible case
    if Length(bl) = 1 and bl[1] = 1 then return qq - 1; fi;

    # get partition and conjugate
    l := [];
    for i in Reversed([1..Length(bl)]) do
        Append(l, List([1..bl[i]], x -> i));
    od;
    k := ConjugatePartition(l);
    s := Length(l);

    # get sum and exponent
    a := HallSumByPartition(k, qq);
    b := Sum(List([1..s], x -> l[x]*(2*x-1)));

    return (qq^b)*a;
end;

#############################################################################
##
#F CentralizerOfType( <type> ) 
##
CentralizerOfType := function( type )
    local d, b, c, i;
    d := List(type, x -> x[2]);
    b := List(type, x -> PartitionChangeBack(x[1]));
    c := 1;
    for i in [1..Length(d)] do
        c := c * CentralizerOfBlock( b[i], d[i] );
    od;
    return c;
end;

#############################################################################
##
#F ClassLengthOfType( <gl>, <type> ) 
##
ClassLengthOfType := function( gl, type )
    return gl/CentralizerOfType(type);
end;

#############################################################################
##
#F GetJNFTypes( <n> ) 
##
GetJNFTypes := function( n )
    local t, i, q, o;

    t := JNFTypes(n);
    q := NameToIndet("q");
    o := Product(List([0..n-1], x -> (q^n-q^x)));
    for i in [1..Length(t)] do
        t[i].clen := ClassLengthOfType(o, t[i].type);
    od;
    return t;
end;

#############################################################################
##
#F ElementaryType
##
ElementaryType := function(n)
    local t, q, o;
    t := rec( type := List([1..n], x -> [[1],1]));
    q := NameToIndet("q");
    o := Product(List([0..n-1], x -> (q^n-q^x)));
    t.clen := ClassLengthOfType(o, t.type);
    t.ss := true;
    return t;
end;
     

