############################################################################
##
#f ApplyAction
##
ApplyAction := function( jnfs, res, act, n )
    return act( jnfs, res, n);
end;


#############################################################################
##
#F CountFixedPoints( <type>, <subtype>, <dim>, <k> )
##
CountFixPoints := function( types, n, K )
    local fixlist, typlist, q, char, fix, p, i, j, c, t;
    
    fixlist := [];
    typlist := [];
    char    := ELIST[n];
    q       := NameToIndet("q");

    fix := List( [1..Length(types)], x -> 
            List( [1..Length(types[x])], y -> 
             List( [1..Length(types[x][y])], z -> [] )));
              
    for i in [1..Length(types)] do
        for j in [1..Length(types[i])] do
            for c in [1..Length(char)] do
                # Get Blks und Degs of type i with subtype j at char c
                t    := [ List( types[i][j][c], x -> x[1] ),
                          List( types[i][j][c], x -> x[2] ) ];
                t[1] := List( t[1], x -> PartitionChangeBack(x) );
                p    := Position( typlist, t );
                if IsBool(p) then
                    fix[i][j][c] := List(K, k -> FixedPointsByDecom( t[1], t[2], q, k));
                    Add( typlist, t );
                    Add( fixlist, fix[i][j][c] );
                else
                    fix[i][j][c] := fixlist[p];
                fi;
            od;            
        od;
    od;
    return fix;             
end;


#############################################################################
##
#F DeterminePorcPoly( <jnfs>, <types>, <fixpoints>, <dim> )
##
DeterminePorcPoly := function( jnfs, res, fix, n )
    local pol, gl, q, i, j, c, k;
    q   := NameToIndet("q");
    pol := List( fix[1][1], x -> List( fix[1][1][1], y -> 0*q ));
    gl  := Product( List( [0..n-1], x -> (q^n-q^x) ) );

    for i in [1..Length(fix)] do
        for j in [1..Length(fix[i])] do
            for c in [1..Length(fix[i][j])] do
                for k in [1..Length(fix[i][j][c])] do
                    pol[c][k] := pol[c][k] + 
                                 jnfs[i].clen * res[i][j].enum * fix[i][j][c][k];
                od;
            od;
        od;
    od;
    
    return pol/gl;  
end;


#############################################################################
##
#F NumberOfClassTwoAlgebras( <n>, <act>, <k> )
##
NumberOfClassTwoAlgebras := function( arg )
    local n, act, k, jnfs, res, types, fix, f, i, l;

    # Interprete input
    n := arg[1];
    act := arg[2];
    
    Info(InfoCountAlg,1, "Determine exceptional chars");
    if not IsBound(ELIST[n]) then
        ELIST[n] := ExceptionalPrimesAssociative(n);
    fi;
    
    # determine possible k's
    Info(InfoCountAlg,1, "Determine or catch extension dimensions");
    if Length(arg)=2 and NameFunction( act ) = "Kronecker" then
        if IsInt(n/2) then l := n^2/2; else l := (n^2-1)/2; fi;
        k := [1..l];
    elif Length(arg)=2 and NameFunction(act) = "WedgeType" then 
        if IsInt(n*(n-1)/4) then l:=n*(n-1)/4; else l:=QuoInt(n*(n-1),4)+1; fi;
        k := [1..l];
    elif Length(arg)=3 then 
        k := [arg[3]];
    fi;

    # Determine all types 
    Info(InfoCountAlg,1, "Determine jnf types in GL");
    jnfs := GetJNFTypes( n );

    # Determine all actions
    Info(InfoCountAlg,1, "Determine equations for ",Length(jnfs)," types ");
    res := List(jnfs, x -> 0);
    for i in [1..Length(jnfs)] do
        if jnfs[i].ss = true then 
            Info(InfoCountAlg, 2,"  Consider type ",i);
            res[i] := DetermineEqns( jnfs[i], act );
        else
            res[i] := ExtractEqns(jnfs[jnfs[i].ss],jnfs[i],res[jnfs[i].ss]);
        fi;
    od;

    # Determine all subtypes 
    Info(InfoCountAlg,1, "Apply action to ",Length(jnfs)," types");
    types := ApplyAction( jnfs, res, act, n );

    # Count Fixpoints
    Info(InfoCountAlg,1, "Determine fixed points in dims: ",k);
    fix := CountFixPoints( types, n, k ); 

    # Determine PORC functions
    Info(InfoCountAlg,1, "Determine PORC poly in dims: ",k);    
    f := DeterminePorcPoly( jnfs, res, fix, n ); 

    #Simplification
    Info(InfoCountAlg,1, "Simplify PORC polys");    
    f := SimplifyPorcRatFun( f, n);
    
    if Length(k) = 1 then return f[1]; else return f; fi;
end;
