#############################################################################
##
#F PorcPolyByExternalRepresentation( <ext> )
##
#   Given an external representation <ext> of a PORC function, this function
#   makes the external representation to a PORC function in the sense of 
#   this package and uses indeterminants whose internal numbers represent 
#   the gcd's they stand for.
#
PorcPolyByExternalRepresentation := function( ext )
    local i, j, l, ind, v, s;
    ind := [];
    l := Length(ext);
    for i in [1,3..l-1] do
        for j in [1,3..Length(ext[i])-1] do
            Add(ind, ext[i][j]);
        od;
    od;
    ind := DuplicateFreeList(ind);
    for i in ind do
        j := NumbToName( i );
        if j = "q" then 
            v := Indeterminate(Rationals, OFFSET_VARS); 
            if not HasName(v) then SetName(v,"q"); fi;
        elif Length(j)=2 then 
            v := Indeterminate(Rationals, NameToNumb(j));
            if not HasName(v) then 
                s := Concatenation("(q-",String(j[2] mod j[1]),",",String(j[1]),")");
                SetName(v,s); 
            fi;
        fi;
    od;
    return PolynomialByExtRep( RationalFunctionsFamily(FamilyObj(1)), ext );
end;




#############################################################################
##
#F PorcToExt( <porc>[, <file>] )
##
#   Given a porc function <porc> (that may not be a PORC polynomial in the 
#   common sense, but may have a non vanishing denominator), an external
#   representation of <porc> is given as pair: The first entry is the 
#   external representation of the numerator, the second entry is the
#   external representation of the denominator.
#   Instead of a single porc function <porc> a list of porc functions or a
#   list of list of porc function may be given.
#   If a second argument <file> is given, where <file> is the name of a
#   readable file is, the external representation is written into that file.
#
PorcToExt := function( arg )
    local d;
    d := NestingDepthA( arg[1] );
    if Length(arg) = 1 then
        if d = 0 then
            return [ ExtRepNumeratorRatFun(arg[1]), 
                     ExtRepDenominatorRatFun(arg[1]) ];
        elif d = 1 then
            return List( arg[1], x -> [ ExtRepNumeratorRatFun(x), 
                                        ExtRepDenominatorRatFun(x) ]);
        elif d = 2 then
            return List( arg[1], x -> List( x, y -> [ ExtRepNumeratorRatFun(y), 
                                                      ExtRepDenominatorRatFun(y) ]));
        else
            Print("Nesting depth of given PORC function too large.\n");
            return fail;
        fi;
    else
        if d = 0 then
            PrintTo(arg[2], "ext := \n",[ ExtRepNumeratorRatFun(arg[1]), 
                                          ExtRepDenominatorRatFun(arg[1]) ],";");
            return true;
        elif d = 1 then
            PrintTo(arg[2], "ext := \n",List( arg[1], x -> [ ExtRepNumeratorRatFun(x), 
                                                             ExtRepDenominatorRatFun(x) ]),";");
            return true;
        elif d = 2 then
            PrintTo(arg[2], "ext := \n",List( arg[1], x -> 
                                            List( x, y -> [ ExtRepNumeratorRatFun(y), 
                                                            ExtRepDenominatorRatFun(y) ])),";");
            return true;
        else
            Print("Nesting depth of given PORC function too large.\n");
            return fail;
        fi;
    fi;
end;




#############################################################################
##
#F PorcByExt( <ext> )
##
#   Given an external representation <ext> of a porc function as pair with 
#   first entry being the numerator and the second entry being the denominator,
#   a porc function in the sense of this package is returned.
#   Note: Instead of a single external representation, a list of external 
#         representations or a list of lists of external representations may be
#         given instead. Then a second argument of the nesting depth (hence
#         '1' or '2') must be given.
#
PorcByExt := function( arg )
    local porc, d;
    porc := arg[1];
    if Length(arg) = 2 then
        d := arg[2];
    else
        d := 0;
    fi;
    if d = 0 then 
        return PorcPolyByExternalRepresentation(porc[1])/PorcPolyByExternalRepresentation(porc[2]);
    elif d = 1 then
        return List( porc, y -> PorcPolyByExternalRepresentation(y[1])/PorcPolyByExternalRepresentation(y[2]));
    elif d = 2 then
        return List( porc, x -> List( x, y -> PorcPolyByExternalRepresentation(y[1])/PorcPolyByExternalRepresentation(y[2])));
    else
        Print("Nesting depth of given PORC function too large.\n");
        return fail;
    fi;
end;
