#
# If i = p1^e1 *..* pr^er, then gcd(q-j, i) = prod gcd(q-j,pi^ei)
# Use this to eliminate gcd(q-j,i) for composite i
# 
# Let f(i) = sum_{a=0}^{i-1} gcd(a,i). Then 
#    * f(p^e) = p^(e-1) * ((e+1)p - e)
#    * f(xy) = f(x)f(y) for coprime x and y
# and gcd(q-(i-1), i) = f(i) - sum_{j=0}^{i-2} gcd(q-j,i) 
#


GcdSum := function(i)
    local a, e, b;
    if i <= 1 then return false; fi;
    e := Collected(Factors(i));
    b := Product(List(e, q -> q[1]^(q[2]-1) * ((q[2]+1)*q[1] - q[2])));
    return b;
    #a := Sum(List([0..i-1], x -> Gcd(i,x)));
    #return a=b;
end;


SimplifyPorcPoly := function( poly )
    local q, vars, nams, g, k, i, j, subs, v, w, f;

    q := NameToIndet("q");
    vars := Filtered( IndetsOfPorcPoly(poly), x -> x <> q );
    nams := List(vars, x -> IndetToName(x));

    # step 1: reduce to prime-power i
    for k in [1..Length(nams)] do
        g := nams[k]; i := g[1]; j := g[2];
        f := Collected(Factors(i));
        if Length(f) > 1 then 
            subs := List(f, x -> NameToIndet([x[1]^x[2], j]));
            poly := Value(poly, [vars[k]], [Product(subs)]);
            vars[k] := false;
            nams[k] := false;
        fi;
    od;

    # set up for step 2
    vars := Filtered(vars, x -> x <> false);
    nams := Filtered(nams, x -> x <> false);

    # step 2: reduce by Gcd sum's
    for k in [1..Length(nams)] do
        g := nams[k]; i := g[1]; j := g[2];
        if j = i-1 then 
            v := GcdSum(i);
            w := List([0..i-1], x -> NameToIndet([i,x]));
            poly := Value(poly, [w[i]], [v-Sum(w{[1..i-1]})]);
        fi;
   od;

   return poly;                 
end;


EvaluateGcdByChar := function( name, p )
    local i, j, f, r, h, s;

    # catch info 
    i := name[1];
    j := name[2];

    # assume that i is prime power
    if not IsPrimePowerInt(i) then return false; fi;
    f := Factors(i);
    r := f[1];
    s := Length(f);

    if r = p and j = 0 then 
        Info(InfoCountAlg,3,"    need at least ",p,"^",s);
        return i;
    elif r = p then 
        h := Length(Filtered(Factors(j), x -> x = p));
        Info(InfoCountAlg,3,"    need at least ",p,"^",h+1);
        return p^h;
    elif r <> p and j = 0 then 
        return 1;
    elif r <> p and IsInt(j/r) then 
        return 1;
    fi;
 
    # nothing we can do
    return NameToIndet( name );
end;


EvaluateGcdNoChar := function( name, pr )
    local i, j, f, r, h, s;

    # catch info 
    i := name[1];
    j := name[2];

    # assume that i is prime power
    if not IsPrimePowerInt(i) then return false; fi;
    f := Factors(i);
    r := f[1];
    s := Length(f);

    if r in pr and IsInt(j/r) then 
        return 1;
    fi;

    # nothing we can do
    return NameToIndet( name );
end;


SimplifyPorcPolyByChar := function( poly, p )
    local q, vars, nams, k, e, vals;

    q := NameToIndet("q");
    poly := SimplifyPorcPoly(poly);
    vars := Filtered( IndetsOfPorcPoly(poly), x -> x <> q );
    nams := List(vars, x -> IndetToName(x));

    for k in Reversed([1..Length(nams)]) do
        e := EvaluateGcdByChar(nams[k], p);
        vals := vars[k];
        if IsInt(e) then 
            vals := e; 
        #elif nams[k] = [9,7] and p = 2  then
        #    vals := -3/4*NameToIndet("q")^0 - 1/4*NameToIndet([9,4]);
        #elif nams[k] = [9,7] and p <> 2  then
        #    vals := -3*NameToIndet("q")^0 + 5*NameToIndet([9,4]) - NameToIndet([9,1]);
        #elif nams[k] = [16,13] and p <> 2 then
        #    vals := -24*NameToIndet("q")^0 + 8*NameToIndet([2,0]) + 40/3*NameToIndet([4,1]) - 8/3*NameToIndet([8,1]);
        #    vals := vals - 1/3*NameToIndet([16,1]) - NameToIndet([16,5]) - 1/3*NameToIndet([16,9]); 
        #    Print("Vereinfachung bei char ",p,"\n");      
        elif nams[k] = [16,13] and p <> 2 then 
            vals := (-2*NameToIndet("q")^0 -NameToIndet([16,5])+7/2*NameToIndet([8,5])-1/2*NameToIndet([8,1]));
        elif nams[k] = [16,9] and p <> 2 then 
            vals := (-2*NameToIndet("q")^0 -NameToIndet([16,1])-1/2*NameToIndet([8,5])+7/2*NameToIndet([8,1]));  
        #elif nams[k] = [9,7] and p <> 2 then
        #    vals := -3*NameToIndet("q")^0+5*NameToIndet([9,4])-NameToIndet([9,1]);
        elif nams[k] = [8,5] and p <> 2 then 
            vals := (4*NameToIndet([4,1]) - 4 - NameToIndet([8,1]));
        fi;
        
        if not vals = vars[k] then
            poly := Value(poly, [vars[k]], [vals]);
        fi; 
    od;

    return poly;
end;


SimplifyPorcPolyNoChar := function( poly, pr )
    local q, vars, nams, k, e, vals;

    if not 2 in pr then Error("this is not expected"); fi;

    q := NameToIndet("q");
    poly := SimplifyPorcPoly(poly);
    vars := Filtered( IndetsOfPorcPoly(poly), x -> x <> q );
    nams := List(vars, x -> IndetToName(x));

    for k in [1..Length(nams)] do
        e := EvaluateGcdNoChar(nams[k], pr);
        vals := vars[k];
        if IsInt(e) then 
            vals := e; 
        #elif nams[k] = [9,7] then
        #    vals := -3*NameToIndet("q")^0 + 5*NameToIndet([9,4]) - NameToIndet([9,1]);
        #elif nams[k] = [16,13] then
        #    vals := -24*NameToIndet("q")^0 + 8*NameToIndet([2,0]) + 40/3*NameToIndet([4,1]) - 8/3*NameToIndet([8,1]);
        #    vals := vals - 1/3*NameToIndet([16,1]) - NameToIndet([16,5]) - 1/3*NameToIndet([16,9]);     
        #    Print("Vereinfachung bei nochar \n"); 
        elif nams[k] = [16,13] then 
            vals := (-2*NameToIndet("q")^0 -NameToIndet([16,5])+7/2*NameToIndet([8,5])-1/2*NameToIndet([8,1]));
        elif nams[k] = [16,9] then 
            vals := (-2*NameToIndet("q")^0 -NameToIndet([16,1])-1/2*NameToIndet([8,5])+7/2*NameToIndet([8,1]));  
        #elif nams[k] = [9,7] then
        #    vals := -3*NameToIndet("q")^0+5*NameToIndet([9,4])-NameToIndet([9,1]);
        elif nams[k] = [8,5] then 
            vals := (4*NameToIndet([4,1]) - 4 - NameToIndet([8,1]));
        fi;
        
        if not vals = vars[k] then
            poly := Value(poly, [vars[k]], [vals]);
        fi;
    od;

    return poly;
end;


RemainderOfPolynomialDivision := function( num, den)
    local pnum, lnum, pden, lden, rem;
    pden := PolynomialCoefficientsOfPolynomial( den, NameToIndet("q"));
    lden := Length(pden);
    rem  := 0*NameToIndet("q");
    repeat
        pnum := PolynomialCoefficientsOfPolynomial( num-rem*den, NameToIndet("q") );
        lnum := Length(pnum);
        rem  := rem + pnum[lnum]/pden[lden]*NameToIndet("q")^(lnum-lden);
    until lnum-lden = 0;
    return num-den*rem;
end;
 
         
ReduceRemainderOfPolynomialDivision := function( porc )
    local num, den ,rem, ind, mul, pol, val, i;
    num := NumeratorOfRationalFunction( porc );
    den := DenominatorOfRationalFunction( porc );
    if IsConstantRationalFunction(den) then
        return porc;
    fi;
    rem := RemainderOfPolynomialDivision( num, den );
    ind := Filtered( List( IndetsOfPorcPoly( rem ), 
                     x -> IndetToName(x) ), y -> not IsString(y));
    mul   := Lcm( List( ind, x -> x[1] )); 
    pol := PolynomialCoefficientsOfPolynomial( rem, NameToIndet("q") );
    for i in [1..Length(pol)] do
        val := Filtered( [0..mul], x -> not ValueOfPorcPoly( pol[i], x ) = 0 );
        if not val = [] then
            Info(InfoCountAlg,4,"    The remainder does not vanish (coefficient ",i,").");
            pol[i] := false;
        else
            pol[i] := true;
        fi;
    od;
    if ForAll( pol, x -> x ) then
        return (num-rem)/den;
    else
        Info(InfoCountAlg, 2,"  Remainder of polynomial division does not vanish.");
        return porc;
    fi;  
end; 


SimplifyPorcRatFun := function( f, n )
    local char, i, c, a, p, g, d; 
    char := ELIST[n];
    for i in [1..Length(f[1])] do
        a := NumeratorOfRationalFunction(f[1][i]);
        a := SimplifyPorcPolyNoChar( a, char{[2..Length(char)]});
        f[1][i] := a / DenominatorOfRationalFunction(f[1][i]);
    od;

    for i in [1..Length(f[1])] do
        for c in [2..Length(char)] do
            p := char[c];
            g := f[c][i] - f[1][i];
            a := NumeratorOfRationalFunction(g);
            a := SimplifyPorcPolyByChar( a, p );
            g := a/DenominatorOfRationalFunction(g);
            d := (NameToIndet([p, 0]) - 1)/(p-1);
            f[1][i] := f[1][i] + d * g; 
        od;
    od;
    return List( f[1], x -> ReduceRemainderOfPolynomialDivision(x) );
end;
